@@ -275,18 +275,18 @@ static glong ORBit_get_union_switch(CORBA_TypeCode tc, gpointer *val, gboolean u
     case CORBA_tk_ulong:
     case CORBA_tk_long:
     case CORBA_tk_enum:
-	retval = *(CORBA_long *)val;
+	retval = *(CORBA_long *)*val;
 	if(update) val += sizeof(CORBA_long);
 	break;
     case CORBA_tk_ushort:
     case CORBA_tk_short:
-	retval = *(CORBA_short *)val;
+	retval = *(CORBA_short *)*val;
 	if(update) val += sizeof(CORBA_short);
 	break;
     case CORBA_tk_char:
     case CORBA_tk_boolean:
     case CORBA_tk_octet:
-	retval = *(CORBA_octet *)val;
+	retval = *(CORBA_octet *)*val;
 	if(update) val += sizeof(CORBA_char);
 	break;
     case CORBA_tk_alias:
@@ -313,7 +313,7 @@ ORBit_get_union_tag(CORBA_TypeCode union_tc,
     int i;
     CORBA_TypeCode retval = CORBA_OBJECT_NIL;
 
-    discrim_val = ORBit_get_union_switch(union_tc->discriminator, *val, TRUE);
+    discrim_val = ORBit_get_union_switch(union_tc->discriminator, val, TRUE);
 
     for(i = 0; i < union_tc->sub_parts; i++) {
 	if(i == union_tc->default_index)
@@ -464,6 +464,8 @@ ORBit_demarshal_allocate_mem(CORBA_TypeCode tc, gint nelements)
     size_t block_size;
     gpointer retval = NULL;
 
+    if(!nelements) return retval;
+
     block_size = ORBit_gather_alloc_info(tc);
 
     if(block_size) {
@@ -616,6 +618,7 @@ ORBit_demarshal_value(GIOPRecvBuffer *buf,
 	else
 	    *(char **)*val = buf->cur;
 	*val = ((guchar *)*val) + sizeof(CORBA_char *);
+	buf->cur += i;
 	break;
     case CORBA_tk_sequence:
 	{
